# coding=utf-8

"""
@fileName       :   simple_pref_test.py
@data           :   2024/4/19
@author         :   jiangmenggui@hosonsoft.com
"""
import functools
import json
import math
import os.path
import queue
import random
import sys
import time
from collections import defaultdict
from concurrent.futures import ThreadPoolExecutor
from inspect import isfunction
from pathlib import Path
from typing import NamedTuple, Callable

from . import logger
from .color import Color
from .console_table import ConsoleTable
from .decorators import catch_exception

data: queue.Queue["TaskResult"] = queue.Queue()
start_time = time.time()
end_time: float | None = None


class TaskResult(NamedTuple):
    name: str
    start: float
    end: float
    message: str = ""
    success: bool = True

    @property
    def use_time(self):
        return self.end - self.start


def task(name: str | Callable = '', weight=1, recode_result=True):
    """
    测试任务
    :param name: 任务名称 或 任务函数
    :param weight: 任务执行权重
    :param recode_result: 是否记录结果
    :return:
    """
    if isinstance(name, Callable):
        task_name = name.__name__

        @functools.wraps(name)
        def inner(*args, **kwargs):
            t1 = time.time()
            try:
                name(*args, **kwargs)
                t2 = time.time()
                if recode_result:
                    data.put(TaskResult(name=task_name, start=t1, end=t2))
            except AssertionError as e:
                t2 = time.time()
                if recode_result:
                    data.put(TaskResult(name=task_name, start=t1, end=t2, message=f'断言失败：{str(e)}', success=False))
                logger.error(f'{task_name}失败：{str(e)}', stacklevel=2)
            except Exception as e:
                t2 = time.time()
                if recode_result:
                    data.put(TaskResult(
                        name=task_name, start=t1, end=t2, message=f'{e.__class__.__name__}: {str(e)}', success=False
                    ))
                logger.exception(f'{task_name}错误：{e}', stacklevel=2)

        inner.is_task = True
        inner.weight = 1
        inner.name = task_name
        return inner
    if not isinstance(weight, int) or weight < 0:
        raise ValueError("任务权重（weight参数）必须为大于0的整数")

    def outer(func):
        task_name = name or func.__name__

        @functools.wraps(func)
        def inner(*args, **kwargs):
            t1 = time.time()
            try:
                func(*args, **kwargs)
                t2 = time.time()
                if recode_result:
                    data.put(TaskResult(name=task_name, start=t1, end=t2))
            except AssertionError as e:
                t2 = time.time()
                if recode_result:
                    data.put(TaskResult(name=task_name, start=t1, end=t2, message=f'断言失败：{str(e)}', success=False))
                logger.error(f'{task_name}失败：{str(e)}', stacklevel=2)
            except Exception as e:
                t2 = time.time()
                if recode_result:
                    data.put(TaskResult(
                        name=task_name, start=t1, end=t2, message=f'{e.__class__.__name__}: {str(e)}', success=False
                    ))
                logger.exception(f'{task_name}错误：{e}', stacklevel=2)

        inner.is_task = True
        inner.weight = weight
        inner.name = task_name
        return inner

    return outer


# 只是用来标记task，不会记录最终的执行耗时结果
mark_task = functools.partial(task, recode_result=False)


def show_result():
    result: dict[str, dict] = {}
    rps: dict[str, dict[str, int]] = {
        time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(i)): defaultdict(int) for i in
        range(int(start_time), int(end_time) + 1)}
    qps: dict[str, dict[str, int]] = {
        time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(i)): defaultdict(int) for i in
        range(int(start_time), int(end_time) + 1)}
    response: dict[str, dict[str, list]] = {
        time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(i)): defaultdict(list) for i in
        range(int(start_time), int(end_time) + 1)}
    while not data.empty():
        row = data.get()
        t1 = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(row.start))
        t2 = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(row.end))
        # logger.info(f'{t1=}, {t2=}')
        if t2 not in rps:
            rps[t2] = defaultdict(int)
        if t1 not in qps:
            qps[t1] = defaultdict(int)

        if t2 not in response:
            response[t2] = defaultdict(list)
        qps[t1][row.name] += 1
        response[t2][row.name].append(row.use_time)
        if row.success:
            rps[t2][row.name + '(成功)'] += 1
        else:
            rps[t2][row.name + '(失败)'] += 1
        if row.name not in result:
            result[row.name] = {
                "NAME": row.name,
                "START_TIME": row.start,
                "END_TIME": row.end,
                "USE_TIME": [row.use_time],
                "SUCCESS": int(bool(row.success))
            }
        else:
            result[row.name]['START_TIME'] = min(result[row.name]['START_TIME'], row.start)
            result[row.name]['END_TIME'] = max(result[row.name]['END_TIME'], row.end)
            result[row.name]['USE_TIME'].append(row.use_time)
            result[row.name]['SUCCESS'] += int(bool(row.success))
    table_data = []
    for value in result.values():
        use_time = sorted(value["USE_TIME"])
        table_data.append({
            "任务名称": value['NAME'],
            "执行次数": len(use_time),
            "错误次数": len(use_time) - value['SUCCESS'],
            "成功率": f"{value['SUCCESS'] / len(use_time):.4%}",
            "中位数响应": f"{int(use_time[int(len(use_time) * 0.5)] * 1000)}ms",
            "90%响应": f"{int(use_time[int(len(use_time) * 0.9)] * 1000)}ms",
            "95%响应": f"{int(use_time[int(len(use_time) * 0.95)] * 1000)}ms",
            "平均响应": f"{int(sum(value['USE_TIME']) * 1000 / len(value['USE_TIME']))}ms",
            "最小响应": f"{int(use_time[0] * 1000)}ms",
            "最大响应": f"{int(use_time[-1] * 1000)}ms",
            "吞吐量（RPS）": f'{value["SUCCESS"] / (end_time - start_time):.2f}/s'
        })
    table = ConsoleTable(table_data, caption="性能测试结果")
    print(table)
    return table, table_data, rps, qps, response


class TaskNotFoundError(ValueError): ...


class PrefRunner:
    """
    测试性能
    :param modules: 测试模块
    :param virtual_users: 虚拟用户数
    :param user_add_interval: 用户增加间隔
    :param run_seconds: 测试时间
    :param pre_task: 每秒执行的任务数量，和virtual_users参数互斥，该参数不为空时，则按照每秒执行的任务数量来执行
    :param save_result_directory: 保存结果目录，默认在当前目录下的simple_pref_test_result目录
    """

    def __init__(
            self,
            __name: str,
            *modules,
            has_main_module=True,
            virtual_users=10,
            user_add_interval=0.1,
            pre_task: float | int = None,
            run_seconds=10,
            save_result_directory='./simple_pref_test_result',
    ):
        self.name = __name
        self.tasks = []
        if has_main_module:
            modules = (*modules, sys.modules['__main__'])
        for module in modules:
            for v in module.__dict__.values():
                if isfunction(v) and getattr(v, 'is_task', False):
                    self.tasks.extend((v for _ in range(getattr(v, 'weight', 1))))
        if not self.tasks:
            raise TaskNotFoundError('没有识别到测试任务！')
        self.run_seconds = run_seconds
        self.user_add_interval = user_add_interval
        self.virtual_users = virtual_users
        self.save_result_directory = os.path.abspath(save_result_directory)

        self.pool = ThreadPoolExecutor(1000)
        self._start_time = 0
        self.pre_task = pre_task
        self.running = True

    def run_task(self, *args, **kwargs):
        if self.pre_task:
            if self.pre_task >= 1:
                self._run_task_with_pre_task(*args, **kwargs)
            elif 0 < self.pre_task < 1:
                self._run_task_with_pre_task_2(*args, **kwargs)
            else:
                raise ValueError(f'pre_task参数必须>0（实际为{self.pre_task}）')
        else:
            self._run_task_with_virtual_users(*args, **kwargs)

    def _run_task_with_virtual_users(self, *args, **kwargs):
        def run(*args, **kwargs):  # noqa
            while time.time() - self._start_time < self.run_seconds and self.running:
                random.choice(self.tasks)(*args, **kwargs)

        for _ in range(self.virtual_users):
            self.pool.submit(run, *args, **kwargs)
            time.sleep(self.user_add_interval)

    def _run_task_with_pre_task(self, *args, **kwargs):

        def run(*args, **kwargs):  # noqa
            random.choice(self.tasks)(*args, **kwargs)

        def submit(k):
            for _ in range(k):
                self.pool.submit(run, *args, **kwargs)

        x, y = divmod(self.pre_task, 9)
        ts = [x] * 9
        exe_status = [False] * 9
        for i in range(y):
            ts[i] += 1
        pre_second = int(time.time() * 10)
        while True:
            now = time.time()
            if now - self._start_time >= self.run_seconds or not self.running:
                break
            now = int(now * 10)
            index = now - pre_second
            if index >= 10:
                pre_second = now
                exe_status = [False] * 9
                submit(ts[0])
                exe_status[0] = True
            elif index < 0:
                raise ValueError('时钟回拨，无法选择任务！')
            else:
                for i in range(index):
                    if exe_status[i]:
                        continue
                    submit(ts[i])
                    exe_status[i] = True

    def _run_task_with_pre_task_2(self, *args, **kwargs):  # pre_task小于1的时候表示每次等待事件必须超过1秒
        interval = math.ceil(1 / self.pre_task)

        def run(*args, **kwargs):  # noqa
            random.choice(self.tasks)(*args, **kwargs)

        while time.time() - self._start_time < self.run_seconds and self.running:
            self.pool.submit(run, *args, **kwargs)
            time.sleep(interval)

    @catch_exception()
    def start(self, *args, **kwargs, ):
        global start_time
        global end_time
        start_time = self._start_time = time.time()
        start_time_str = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(self._start_time))
        print(f"开始测试：{Color.green(self.name)}")
        if self.pre_task:
            print(
                f"{Color.yellow('==========测试任务启动参数==========')}\n"
                f"   {Color.thin_magenta('每秒任务数')} : {self.pre_task}\n"
                f"     {Color.thin_magenta('运行时间')} : {self.run_seconds}s\n"
                f"     {Color.thin_magenta('任务总数')} : {len(set(self.tasks))}\n"
                f"     {Color.thin_magenta('启动时间')} : {start_time_str}\n"
                f"{Color.yellow('====================================')}\n"
            )
        else:
            print(
                f"{Color.yellow('==========测试任务启动参数==========')}\n"
                f"   {Color.thin_magenta('并发线程数')} : {self.virtual_users}\n"
                f" {Color.thin_magenta('线程启动间隔')} : {self.user_add_interval}s\n"
                f"     {Color.thin_magenta('运行时间')} : {self.run_seconds}s\n"
                f"     {Color.thin_magenta('任务总数')} : {len(set(self.tasks))}\n"
                f"     {Color.thin_magenta('启动时间')} : {start_time_str}\n"
                f"{Color.yellow('====================================')}\n"
            )
        try:
            self.run_task(*args, **kwargs)
            self.pool.shutdown(wait=True)
        finally:
            end_time = time.time()
            table, table_data, rps, qps, response = show_result()
            end_time_str = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time()))
            end_str = f'\n{Color.green("测试完成！")}[完成时间{Color.thin_cyan(end_time_str)}]'
            print('\n' + end_str)
            self._save(table_data, rps, qps, response)

    def _save_html(self, file, table, rps, qps, response):

        def init_tps_data(tps):
            tps = sorted(tps.items())
            keys = set()
            for _, v in tps:
                for k in v:
                    keys.add(k)
            values = {k: [] for k in keys}
            for _, v in tps:
                for k in keys:
                    values[k].append(v.get(k, 0))
            return json.dumps({
                'keys': [k.split(' ')[1] for k, v in tps],
                "values": [{"name": k, "data": v} for k, v in values.items()]
            }, ensure_ascii=False)

        def init_response_data(response):
            response = sorted(response.items())
            keys = set()
            for _, v in response:
                for k in v:
                    keys.add(k)
            values = {k: [] for k in keys}
            for _, v in response:
                for k in keys:
                    v_ = v.get(k, [0])
                    values[k].append(int(sum(v_) / len(v_) * 1000))
            return json.dumps({
                'keys': [k.split(' ')[1] for k, v in response],
                "values": [{"name": k, "data": v} for k, v in values.items()]
            }, ensure_ascii=False)

        def init_table_data(table_data):
            return {"header": list(table_data[0].keys()), "rows": [list(row.values()) for row in table_data]}

        from jinja2 import Environment, FileSystemLoader, select_autoescape

        # 创建一个环境，指定模板文件所在的路径
        env = Environment(
            loader=FileSystemLoader(Path(__file__).parent),
            autoescape=select_autoescape(['html', 'xml'])
        )
        if self.pre_task:
            run_arguments = [
                f"每秒任务数 : {self.virtual_users}"
            ]
        else:
            run_arguments = [
                f"并发线程数 : {self.virtual_users}",
                f"线程启动间隔 : {self.user_add_interval}s"
            ]
        with open(Path(__file__).parent / 'echart.js', 'r', encoding='u8') as f:
            echart_js = f.read()

        template = env.get_template('result.html')
        context = {
            "echartJs": echart_js,
            "title": self.name,
            'arguments': [
                *run_arguments,
                f"运行时间 : {self.run_seconds}s",
                f"任务总数 : {len(set(self.tasks))}",
                f"启动时间 : {time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(start_time))}",
                f"结束时间 : {time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(end_time))}"],
            "table": init_table_data(table),
            "rpsData": init_tps_data(rps),
            "qpsData": init_tps_data(qps),
            "responseData": init_response_data(response)
        }
        with open(file, 'w', encoding='u8') as f:
            f.write(template.render(context))

    def _save(self, table, rps, qps, response):
        if not os.path.exists(self.save_result_directory):
            os.mkdir(self.save_result_directory)
        file = os.path.join(self.save_result_directory, f'{self.name}_{time.strftime("%Y%m%d%H%M%S")}')
        self._save_html(file + '.html', table, rps, qps, response)
        print(f'\n结果已保存至：{Color.thin_blue(file + ".html")}')


if __name__ == '__main__':
    print(math.ceil(3 / 10))



import urllib.parse
from http.client import HTTPConnection, HTTPResponse
from json import dumps, loads
from logging import Logger

from requests import Session
from urllib3.filepost import choose_boundary

from lljz_tools import logger
from lljz_tools.color import Color
from lljz_tools.decorators import thread_lock, debug


class HTTPClient(Session):

    def __init__(self, base_url: str = '', *, timeout=120, debug: bool | Logger = False):
        super().__init__()
        self._base_url = base_url.rstrip('/')
        self._timeout = timeout
        self.debug = bool(debug)
        self._logger = debug if isinstance(debug, Logger) else logger

    def request(
            self,
            method,
            url,
            params=None,
            data=None,
            headers=None,
            cookies=None,
            files=None,
            auth=None,
            timeout=None,
            allow_redirects=True,
            proxies=None,
            hooks=None,
            stream=None,
            verify=False,
            cert=None,
            json=None,
    ):
        if not url.startswith('http://') and not url.startswith('https://'):
            url = self._base_url + url
        if self.debug:
            method = method.upper()
            method_color = {
                'POST': Color.yellow, 'GET': Color.green, 'DELETE': Color.red, 'PUT': Color.magenta
            }.get(method, Color.cyan)
            self._logger.info(f'{method_color(method)} - {url}', stacklevel=5)
        if self.debug and params:
            self._logger.info(f'params - {params}', stacklevel=5)
        if self.debug and json:
            try:
                json_data = dumps(json, ensure_ascii=False)
                self._logger.info(f'Payload - {json_data}', stacklevel=5)
            except TypeError as e:
                self._logger.error(f'json dumps error: {e}', stacklevel=5)
        if self.debug and data:
            self._logger.info(f'Form data - {data}', stacklevel=5)

        return super().request(
            method, url,
            params=params,
            data=data,
            headers=headers,
            cookies=cookies,
            files=files,
            auth=auth,
            timeout=timeout,
            allow_redirects=allow_redirects,
            proxies=proxies,
            hooks=hooks,
            stream=stream,
            verify=verify,
            cert=cert,
            json=json,
        )


class FastResponse:

    def __init__(self, response: HTTPResponse):
        self._response = response
        self.status_code = response.status
        self.content = response.read()
        self.text = self.content.decode('utf-8')
        self.headers = response.headers
        self._json = None

    def json(self):
        if not self._json:
            self._json = loads(self.text)
        return self._json

    def __repr__(self):
        return f'<FastResponse [{self.status_code}]>'

    def __bool__(self):
        return self.status_code == 200

    def __str__(self):
        return f'<FastResponse [{self.status_code}]>'


class FastHTTPClient:

    def __init__(self, base_url: str = '', *, timeout=120, debug: bool | Logger = False):
        if base_url.startswith('https://'):
            raise ValueError('FastHTTPClient 不支持https协议')
        self._base_url = urllib.parse.urlparse(base_url)
        self._timeout = timeout
        self.debug = debug
        self._connections: list[list[HTTPConnection | bool]] = []
        self._closed = False
        self.headers = {}
        self.debug = bool(debug)
        self._logger = debug if isinstance(debug, Logger) else logger

    @thread_lock
    def __get_connection(self) -> tuple[int, HTTPConnection]:
        if self._closed:
            raise ValueError("FastHTTPClient 已关闭")
        for index, (connection, status) in enumerate(self._connections):
            if not status:
                self._connections[index][1] = True
                return index, connection
        else:
            self._connections.append([HTTPConnection(self._base_url.netloc, timeout=self._timeout), True])
            return len(self._connections) - 1, self._connections[-1][0]

    def set_token(self, token: str, key='Authorization'):
        if len(token.split(' ')) == 1:
            token = f"Bearer {token}"
        self.headers[key] = token

    @staticmethod
    def _encode_data_to_form_data(data):
        """将dict转换为multipart/form-data"""
        body = b''
        boundary = f'----{choose_boundary()}'
        content_type = f'multipart/form-data; boundary={boundary}'  # noqa
        for key, value in data.items():
            value = "" if value is None else str(value)
            body += f'--{boundary}\r\n'.encode('utf-8')
            body += f'Content-Disposition: form-data; name="{key}"\r\n\r\n'.encode('utf-8')  # noqa
            body += f'{value}\r\n'.encode('utf-8')
        body += f'--{boundary}--'.encode('utf-8')
        return body, content_type

    def request(self, method, url: str, *, params=None, data=None, json=None, headers=None, **kwargs):
        if url.startswith('http://') or url.startswith('https://'):
            raise ValueError('URL不允许重新指定网络地址')
        if data and json:
            raise ValueError('data和json不能同时存在')
        if params:
            url = f'{url}?{urllib.parse.urlencode(params)}'
        if self.debug:
            method = method.upper()
            method_color = {
                'POST': Color.yellow, 'GET': Color.green, 'DELETE': Color.red, 'PUT': Color.magenta
            }.get(method, Color.cyan)
            self._logger.info(
                f'{method_color(method)} - {self._base_url.scheme}://{self._base_url.netloc}{url}', stacklevel=5
            )
        body = None
        if not headers:
            headers = {}
        if json:
            body = dumps(json, ensure_ascii=True)
            if self.debug:
                self._logger.info(f'Payload - {dumps(json, ensure_ascii=False)}', stacklevel=5)
            headers['Content-Type'] = 'application/json;charset=UTF-8'
        if data:
            body, content_type = self._encode_data_to_form_data(data)
            headers['Content-Type'] = content_type
            if self.debug:
                self._logger.info(f'Form data - {data}', stacklevel=5)
        index, conn = self.__get_connection()
        headers.update(self.headers)
        conn.request(method, url, body=body, headers=headers)
        response = FastResponse(conn.getresponse())
        self._connections[index][1] = False
        return response

    def get(self, url, *, params=None, headers=None):
        return self.request('GET', url, params=params, headers=headers)

    def post(self, url, *, data=None, json=None, headers=None):
        return self.request('POST', url, data=data, json=json, headers=headers)

    def put(self, url, *, data=None, headers=None):
        return self.request('PUT', url, data=data, headers=headers)

    def delete(self, url, *, data=None, headers=None):
        return self.request('DELETE', url, data=data, headers=headers)

    def close(self):
        for connection, status in self._connections:
            connection.close()
        self._connections.clear()
        self._closed = True

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()


if __name__ == '__main__':
    from concurrent.futures import ThreadPoolExecutor

    client = FastHTTPClient('http://127.0.0.1:8087', debug=False)

    @logger.catch_exception
    def search():
        for _ in range(10000):
            client.get('/')


    @debug
    def main():
        with ThreadPoolExecutor(max_workers=10) as executor:
            for _ in range(10):
                executor.submit(search)

    main()



import io
import keyword
import logging
import os
import re
import sys
import tokenize
from logging import Formatter as BaseFormatter
from os import PathLike
from pathlib import Path
from typing import TextIO, Callable, Optional, Literal

import better_exceptions
from colorlog import ColoredFormatter as CF  # noqa
from concurrent_log_handler import ConcurrentTimedRotatingFileHandler

from .color import Color
from .decorators import singleton

default_level_colors = {
    "TRACE": "cyan",
    "DEBUG": "blue",
    "SUCCESS": "light_green",
    "WARNING": "yellow",
    "ERROR": "red",
    "CRITICAL": "bold_red",
}

default_secondary_log_colors = {
    "levelname": "level",
    "message": "level",
    "asctime": 'green',
    "name": 'purple',
    "module": 'cyan',
    "funcName": 'cyan',
    "lineno": 'cyan',
}


class ExceptionFormat:

    @staticmethod
    def _tokenize(source):
        # Worth reading: https://www.asmeurer.com/brown-water-python/
        source = source.encode("utf-8")
        source = io.BytesIO(source)

        try:
            yield from tokenize.tokenize(source.readline)
        except tokenize.TokenError:
            return
    @classmethod
    def _formate_code_line(cls, line: str):
        row = ''

        tokens = list(cls._tokenize(line))[1:]

        def add_row(index, val):
            if index == 0:
                return val
            else:
                sep = ' ' * (tokens[index].start[1] - tokens[index - 1].end[1])
                return sep + val

        for i, token in enumerate(tokens):
            if token.type == tokenize.COMMENT:
                row += add_row(i, '\033[37;3m' + token.string + '\033[0m')
            elif token.type == tokenize.NAME:
                # 如果是keyword，则变为紫色
                if token.string in keyword.kwlist:
                    row += add_row(i, '\033[35;1m' + token.string + '\033[0m')
                else:
                    row += add_row(i, token.string)
            elif token.type == tokenize.NUMBER:
                row += add_row(i, '\033[32m' + token.string + '\033[0m')
            elif token.type == tokenize.STRING:
                row += add_row(i, '\033[36m' + token.string + '\033[0m')
            else:
                row += add_row(i, token.string)
        return row + '\n'

    @classmethod
    def _formate_exception_line(cls, line):
        is_code = False

        for inner_line in line.split('\n'):
            if is_code:
                yield cls._formate_code_line(inner_line)
                is_code = False
            elif inner_line.startswith('  File "'):
                obj = re.match(
                    r'^\s+File "(?P<filename>[^"]+)", line (?P<lineno>\d+), in (?P<name>[^"]+)$',
                    inner_line
                )
                if obj:
                    yield (f'\n  \033[31mFile "{obj.group("filename")}", line\033[0m '
                           f'\033[33m{obj.group("lineno")}\033[0m\033[31m, in \033[0m'
                           f'\033[35m{obj.group("name")}\033[0m\n')
                    is_code = True
                else:
                    yield inner_line + '\n'
            else:
                yield '\033[36m' + inner_line + '\033[0m\n'

    @classmethod
    def format_exception(cls, exc_info):
        formatter = better_exceptions.ExceptionFormatter(colored=False)
        for line in formatter.format_exception(*exc_info):
            if line == 'Traceback (most recent call last):\n':
                yield '\033[33;1mTraceback (most recent call last):\n\033[0m'
            elif line == '\nThe above exception was the direct cause of the following exception:\n\n':
                yield '\n\033[31mThe above exception was the direct cause of the following exception:\n\n\033[0m'
            elif line == '\nDuring handling of the above exception, another exception occurred:\n\n':
                yield '\n\033[31mDuring handling of the above exception, another exception occurred:\n\n\033[0m'
            elif not line.startswith(' '):
                error, *message = line.split(':', maxsplit=1)
                line = f'\033[31;1m{error}\033[0m'
                if message:
                    line += f':{"".join(message)}'
                yield line
            else:
                yield from cls._formate_exception_line(line)


class ColoredFormatter(CF):

    def __init__(
            self,
            fmt: Optional[str] = None,
            datefmt: Optional[str] = None,
            log_colors: Optional[dict] = None,
            reset: bool = True,
            secondary_log_colors: Optional[dict] = None,
            level_colors: Optional[dict] = None,
            style='{',
    ) -> None:
        level_colors = level_colors or default_level_colors
        secondary_log_colors = secondary_log_colors or default_secondary_log_colors
        secondary_log_colors = {
            k: ({lv: v for lv in logging._nameToLevel} if v != 'level' else level_colors)  # noqa
            for k, v in secondary_log_colors.items()
        }

        for k in secondary_log_colors.keys():
            if style == '{':
                fmt = re.sub(rf'({{{k}.*?}})', rf'{{{k}_log_color}}\1{{reset}}', fmt)
            elif style == '%':
                fmt = re.sub(rf'(%\({k}.*?\)s)', rf'%({k}_log_color)s\1%(reset)s', fmt)
            elif style == '$':
                fmt = re.sub(rf'(\${{{k}.*?}})', rf'${{{k}_log_color}}\1${{reset}}', fmt)
            else:
                raise ValueError('style only support \'{\' or \'%\' or \'$\'')
        super().__init__(
            fmt=fmt,
            datefmt=datefmt,
            style=style,
            log_colors=log_colors,
            reset=reset,
            secondary_log_colors=secondary_log_colors,
        )

    def format(self, record):
        if record.exc_info:
            if not getattr(record, '_is_better_exception', False):
                record.exc_text = ''.join(ExceptionFormat.format_exception(record.exc_info))
                record._is_better_exception = True
        return super().format(record)


class Formatter(BaseFormatter):

    def format(self, record):
        if record.exc_info:
            if not getattr(record, '_is_better_exception', False):
                record.exc_text = ''.join(ExceptionFormat.format_exception(record.exc_info))
                record._is_better_exception = True
        return re.sub(r'\033\[.+?m', '', super().format(record))


ColoredFormatter.default_msec_format = '%s.%03d'
Formatter.default_msec_format = '%s.%03d'
BaseFormatter.default_msec_format = '%s.%03d'

TRACE = 5
SUCCESS = 25
logging.addLevelName(TRACE, "TRACE")
logging.addLevelName(SUCCESS, "SUCCESS")
default_fmt = '{asctime} | {levelname:<8} | {name} | {module}:{funcName}:{lineno} - {message}'
default_console_format = ColoredFormatter(default_fmt)
default_file_format = Formatter(default_fmt, style='{')
default_format = BaseFormatter(default_fmt, style='{')


@singleton('name')
class ManagerLogger(logging.Logger):

    def catch_exception(self, func: Callable):
        def inner(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as e:
                self.exception(f'{func.__name__}执行错误：[{e.__class__.__name__}]{e}', stacklevel=2)

        return inner

    def _add_console_handler(self, stream=None, level=None, formatter: str | Formatter = None,
                             filters: Callable | list[Callable] = None):
        if stream is None:
            stream = sys.stdout
        handler = logging.StreamHandler(stream)
        if not formatter:
            formatter = default_console_format
        if isinstance(formatter, str):
            formatter = Formatter(formatter)
        handler.setFormatter(formatter)
        handler.setLevel(level)
        if filters is not None:
            if isinstance(filters, Callable):
                filters = [filters]
            for f in filters:
                handler.addFilter(f)
        self.addHandler(handler)

    def _add_file_handler(self, path: str | PathLike, level=None, formatter: str | Formatter = None,
                          filters: Callable | list[Callable] = None):
        handler = ConcurrentTimedRotatingFileHandler(
            path,
            when="midnight",
            backupCount=10,
            encoding="utf-8",
            delay=True,
            maxBytes=10 * 1024 * 1024,
        )
        handler.setLevel(level)
        if not formatter:
            formatter = default_file_format
        if isinstance(formatter, str):
            formatter = Formatter(formatter)
        handler.setFormatter(formatter)
        if filters is not None:
            if isinstance(filters, Callable):
                filters = [filters]
            for f in filters:
                handler.addFilter(f)
        self.addHandler(handler)

    def remove(self, handler=None):
        if handler:
            return self.removeHandler(handler)
        while self.handlers:
            self.handlers.pop()

    def add(self, sink: TextIO | str | PathLike, *, level: int | str = "DEBUG", formatter=None, filters=None):
        if sink == sys.stdout or sink == sys.stderr:
            return self._add_console_handler(sink, level=level, filters=filters, formatter=formatter)
        return self._add_file_handler(sink, level=level, filters=filters, formatter=formatter)

    def success(self, msg, *args, exc_info=None, extra=None, stack_info=False, stacklevel=1):
        return self._log(
            SUCCESS, msg, args, exc_info=exc_info, extra=extra, stack_info=stack_info, stacklevel=stacklevel + 1
        )

    def trace(self, msg, *args, exc_info=None, extra=None, stack_info=False, stacklevel=1):
        return self._log(
            TRACE, msg, args, exc_info=exc_info, extra=extra, stack_info=stack_info, stacklevel=stacklevel + 1
        )


getLogger_raw = logging.getLogger


def _get_loger(name=None):
    __logger = getLogger_raw(name)
    __logger_new = ManagerLogger(__logger.name, __logger.level)
    __logger_new.handlers = __logger.handlers
    __logger_new.filters = __logger.filters
    return __logger_new


logging.getLogger = _get_loger


class LogManager:

    def __init__(
            self, __name, *,
            console_level: int | str = "DEBUG",
            file_path: str = None,
            file_level: int | str | None = "DEBUG",
            file_name: str = 'out.log',
            error_level: int | str | None = "ERROR",
            error_file_name: str = 'error.log',
            fmt: str = default_fmt,
            colorize: bool = True,
    ) -> None:
        logging.root.setLevel(logging.NOTSET)
        self._file_path = self._init_log_file_path(file_path)
        if not self._file_path.exists():
            os.mkdir(self._file_path)
        self.console_level = console_level
        self.file_level = file_level
        self.error_level = error_level
        self._logger: ManagerLogger = _get_loger(__name)
        self.fmt = fmt
        self.colorize = colorize
        self.file_name = file_name
        self.error_file_name = error_file_name

    @staticmethod
    def _init_log_file_path(file_path: str | PathLike = None) -> Path:
        if file_path:
            return Path(file_path)
        if "PYTHONPATH" in os.environ and os.path.exists(os.environ['PYTHONPATH']):
            return Path(os.environ['PYTHONPATH']) / 'logs'
        return Path('/pythonlogs')

    @staticmethod
    def _guess_fmt_style(fmt: str) -> Literal['{', '%', '$']:
        for i in fmt:
            if i == '{':
                return '{'
            if i == '%':
                return '%'
            if i == '$':
                return '$'
        return '{'

    def get_logger(self, reset=False) -> ManagerLogger:
        """

        :param reset: 是否重置全部handler，重置后，将自动改用带彩色的控制台日志，以及进程安全的文件日志
        :return:
        """
        if reset:
            self._logger.remove()
        if not self._logger.handlers:
            style = self._guess_fmt_style(self.fmt)
            file_format = Formatter(self.fmt, style=style)
            if not self.colorize:
                console_format = BaseFormatter(fmt=self.fmt, style=style)
            else:
                console_format = ColoredFormatter(fmt=self.fmt, style=style)
            self._logger.add(sys.stdout, level=self.console_level, formatter=console_format)
            if self.file_level:
                self._logger.add(self._file_path / self.file_name, level=self.file_level, formatter=file_format)
            if self.error_level:
                self._logger.add(self._file_path / self.error_file_name, level=self.error_level, formatter=file_format)
        return self._logger


if __name__ == '__main__':
    logger = LogManager(__name__, console_level=TRACE, file_path='./../logs').get_logger()

    def func(a, b):
        return a / b


    @logger.catch_exception
    def func2():
        return func(1, 0)


    func2()



# coding=utf-8

"""
@fileName       :   __init__.py
@data           :   2024/2/3
@author         :   jiangmenggui@hosonsoft.com
"""
from .attribute_dict.model import AttributeDict, IgnoreCaseAttributeDict

if __name__ == '__main__':
    pass



# coding=utf-8

"""
@fileName       :   model.py
@data           :   2024/2/3
@author         :   jiangmenggui@hosonsoft.com
"""
import re
from itertools import cycle
from types import GenericAlias, UnionType


class _AttributeDictMetaclass(type):

    def __new__(cls, name, bases, attrs, total=False, variable=True, check_type=False):
        """

        :param name:
        :param bases:
        :param attrs:
        :param total: 是否全部字段必填
        :param variable: 是否允许增加字段
        :param check_type: 是否需要检查类型
        """
        if name in ['AttributeDict', 'IgnoreCaseAttributeDict']:
            return type.__new__(cls, name, bases, attrs)
        attrs['__default__'] = {}
        __remove__ = []
        for key, value in attrs.items():
            if key.startswith('__'):  # 跳过双下划綫开头的属性
                continue
            attrs['__default__'][key] = value
            __remove__.append(key)
        for k in __remove__:
            attrs.pop(k)
        attrs['__d_args__'] = (total, variable, check_type)
        return type.__new__(cls, name, bases, attrs)


def _init_value(value, type_):
    try:
        if isinstance(type_, GenericAlias):
            if issubclass(type_.__origin__, dict):
                if not isinstance(value, dict):
                    raise TypeError
                if len(type_.__args__) == 2:
                    return type_.__origin__(
                        {_init_value(v, type_.__args__[0]): _init_value(v, type_.__args__[1]) for k, v in value.items()}
                    )
                elif len(type_.__args__) == 1:
                    return type_.__origin__({k: _init_value(v, type_.__args__[0]) for k, v in value.items()})
                else:
                    return type_.__origin__(value)
            elif issubclass(type_.__origin__, list | tuple | set):
                return type_.__origin__(_init_value(v, t) for t, v in zip(cycle(type_.__args__), value))
            return type_.__origin__(value)
        elif isinstance(type_, UnionType):
            for t in type_.__args__:
                try:
                    return _init_value(value, t)
                except TypeError:
                    continue
            else:
                raise TypeError(f"{value!r} must be {type_!r}")
        return type_(value)
    except ValueError:
        raise TypeError(f"{value!r} must be {type_!r}")
    except TypeError:
        raise TypeError(f"{value!r} must be {type_!r}")


class AttributeDict(dict, metaclass=_AttributeDictMetaclass):

    def __init__(self, __mapper=None, **kwargs):
        super().__init__((__mapper or {}), **kwargs)

        for key, value in self.__annotations__.items():
            key = self._get_key(key)
            if key in self:
                if self.__d_args__[2]:  # 检查类型
                    self[key] = _init_value(self[key], value)
                elif isinstance(value, _AttributeDictMetaclass):
                    self[key] = value(**self[key])
                elif isinstance(value, GenericAlias):
                    if not self[key]:
                        break
                    for t in value.__args__:
                        print('>>>>>', t)
                        if not isinstance(t, _AttributeDictMetaclass):
                            continue
                        try:
                            self[key] = value.__origin__(t(**v) for v in self[key])
                            break
                        except Exception:  # noqa
                            continue
                    # if self[key]:
                    #     self[key] = [value.__args__[0](**v) for v in self[key]]
                continue
            if key not in self.__default__ and self.__d_args__[0]:
                raise ValueError(f"{self.__class__.__name__!r} missing key {key!r}")
            self[key] = self.__default__.get(key, None)

    def _get_key(self, key):
        return key

    def __getattr__(self, item):
        try:
            return self[self._get_key(item)]
        except KeyError:
            if self.__d_args__[1]:
                return None
            raise AttributeError(f"{self.__class__.__name__!r} object has no attribute {item!r}")

    def __setattr__(self, item, value):
        item = self._get_key(item)
        if item not in self and not self.__d_args__[1]:
            raise TypeError(f"{self.__class__.__name__!r} cannot add attribute {item!r}")
        self[item] = value

    def __delattr__(self, item):
        return self.__delitem__(self._get_key(item))

    __show__ = None

    def __str__(self):
        show = self.__show__
        if not show:
            show = self.__annotations__.keys()
        if isinstance(show, str):
            show = re.split(r'[,\s]+', show.strip())
        val = ', '.join(f"{key}={self.__getattr__(key)!r}" for key in show)
        return f'{self.__class__.__name__}({val})'

    __repr__ = __str__


class IgnoreCaseAttributeDict(AttributeDict):

    def _get_key(self, key):
        for k in self.keys():
            if k.lower() == key.lower():
                return k
        return super()._get_key(key)


if __name__ == '__main__':
    class Book(AttributeDict, variable=False, total=True, check_type=True):
        name: str
        author: str
        price: float
        history: set[int]


    class User(IgnoreCaseAttributeDict, total=True, variable=True, check_type=True):
        name: int | str
        books: list[Book]


    user = User(name='tom', books=[{'name': 'python', 'author': 123, 'price': '3.2', 'history': ('1', 2, 3, 2)}])
    # print(user.password)
    user.password = "abc"
    print(user.password)



# coding=utf-8

"""
@fileName       :   __init__.py
@data           :   2024/2/2
@author         :   jiangmenggui@hosonsoft.com
"""
from .log_manager import LogManager

logger = LogManager('my-tools', console_level='DEBUG').get_logger()

if __name__ == '__main__':
    logger.debug('debug')
    pass



# coding=utf-8
from typing import Iterable, Any

from .color import _Color, Color


class ConsoleTable:

    def __init__(self, data: Iterable[dict[str, Any]], max_width=100, caption='',
                 caption_color=Color.yellow, title_color=Color.thin_cyan, colorize=True):
        def init_value(val):
            if isinstance(val, str | _Color):
                return val
            if val is None:
                return ''
            return str(val)
        self.colorize = colorize
        self.caption = caption
        self.caption_color = caption_color
        self.title_color = title_color
        self.data = [{str(k): init_value(v) for k, v in row.items()} for row in data]
        self.header = list(self.data[0].keys()) if data else []
        self.max_width = max_width
        self.col_width = []
        self._table_str = ""
        self.col_width = self._get_widths()
        self._table_str = self.make_table_str() if self.colorize else self.make_table_str_without_color()

    @staticmethod
    def _get_string_width(val: str):
        w = 0
        for v in val:
            if u'\u4e00' <= v <= u'\u9fff' or v in '【】（）—…￥！·、？。，《》：；‘“':
                w += 2
            else:
                w += 1
        return w

    def _get_widths(self):
        """获取列宽度，列宽度为整列数据中的最大数据宽度"""

        col_width = [self._get_string_width(key) for key in self.header]
        for row in self.data:
            for i, key in enumerate(self.header):
                value = row.get(key, '')
                width = min(self._get_string_width(value), self.max_width)
                col_width[i] = max(col_width[i], width)
        return col_width

    def make_table_str(self):
        def format_str(val, width):
            length = self._get_string_width(val)
            left = (width - length) // 2
            right = (width - length) - left
            return f'{" " * left}{val}{" " * right}'

        header = ' | '.join(str(self.title_color(format_str(key, w))) for w, key in zip(self.col_width, self.header))
        if self.caption:
            caption = self.caption_color(format_str(self.caption, sum(self.col_width) + (len(self.col_width) - 1) * 3))
            header = caption + '\n' + header
        rows = [' | '.join(format_str(row.get(key, ""), w) for w, key in zip(self.col_width, self.header)) for row in
                self.data]
        return '\n'.join([header, '=' * (sum(self.col_width) + (len(self.col_width) - 1) * 3)] + rows)

    def make_table_str_without_color(self):
        def format_str(val, width):
            length = self._get_string_width(val)
            left = (width - length) // 2
            right = (width - length) - left
            return f'{" " * left}{val}{" " * right}'

        def get_value(row, key):
            val = row.get(key, "")
            if isinstance(val, _Color):
                return val.raw
            return val

        header = ' | '.join(str(format_str(key, w)) for w, key in zip(self.col_width, self.header))
        if self.caption:
            caption = format_str(self.caption, sum(self.col_width) + (len(self.col_width) - 1) * 3)
            header = caption + '\n' + header
        rows = [' | '.join(format_str(get_value(row, key), w) for w, key in zip(self.col_width, self.header)) for row in
                self.data]
        return '\n'.join([header, '=' * (sum(self.col_width) + (len(self.col_width) - 1) * 3)] + rows)

    def __str__(self):
        return self._table_str

    __repr__ = __str__


if __name__ == '__main__':
    table = ConsoleTable(
        [{'server_name': 'intelligent-platform-product', 'status': Color.green('成功'), 'message': '构建成功'}],
        caption='构建结果',
    )
    print(table)



"""
@file   : c_random.py
@author : jiangmenggui@hosonsoft.com
@data   : 2024/4/2
"""
import enum
import random


class GroupType(enum.Enum):
    """随机字符允许出现的类型"""
    NUMBER = 'integer'  # 数字
    LETTERS = 'letters'  # 字母
    LETTERS_UPPER = 'lettersUpper'  # 大写字母
    LETTERS_LOWER = 'lettersLower'  # 小写字母
    SPECIAL = 'special'  # 特殊符号


def random_upper_or_lower(s: str):
    """将字符中的字母随机转换为大写或者小写"""
    return ''.join(random.choice([i.upper(), i.lower()]) for i in s)


def random_string(size: int | list[int] = 15, groups: list[GroupType | str] = None):
    """
    :param size: 随机得到的字符长度，如果给出一个数组，则表示每种字符类型的长度
    :param groups: 包含的字符类型：number（数字）、letters（全部字母）lettersUpper（大写字母）、lettersLower（小写字母）、special（特殊字符）
    :return:
    """
    groups = groups or ['integer', 'letters']
    groups = [group.value if isinstance(group, GroupType) else group for group in groups]
    if isinstance(size, list):
        return ''.join(random_string(i, [j]) for i, j in zip(size, groups))
    charList = []
    for g in groups:
        if g == 'integer':
            charList.extend(list('0123456789'))
        elif g == 'letters':
            charList.extend(list(chr(i) for i in range(ord('a'), ord('z') + 1)))
            charList.extend(list(chr(i) for i in range(ord('A'), ord('Z') + 1)))
        elif g == 'lettersUpper':
            charList.extend(list(chr(i) for i in range(ord('A'), ord('Z') + 1)))
        elif g == 'lettersLower':
            charList.extend(list(chr(i) for i in range(ord('a'), ord('z') + 1)))
        elif g == 'special':
            charList.extend(list('`~!@#$%^&*()_+-={}|:"?><,./;\'[]\\'))
    random.shuffle(list(set(charList)))
    result = ''.join(random.choice(charList) for _ in range(size))
    while result.startswith('0'):
        result = random_integer(size)
    return result


def random_integer(size: int | list[int]):
    """生成一个指定长度的随机数字"""
    return random_string(size, groups=[GroupType.NUMBER])


def random_string_series(series: int, size=15, groups: list[str | GroupType] = None):
    """
    生成一系列的String，字符前缀都相同，后面的序号不同
    random_string_series(3)
    ['3cO5WiACaDVex9M001', '3cO5WiACaDVex9M002', '3cO5WiACaDVex9M003']  # 一个可能的输出

    :param series: 生成的个数,1~100的整数
    :param size: 字符的长度，实际长度为size+序列号
    :param groups: 随机字符允许出现的类型
    :return:
    """
    series = max(1, min(series, 100))
    s = random_string(size, groups)
    size = max(2, len(str(series)))
    return (f'{s}{i:0>{size}}' for i in range(1, series + 1))


if __name__ == '__main__':
    print(list(random_string_series(3)))
    print(random_integer(3))
    print(random_string(20))
    pass



# coding=utf-8

"""
@fileName       :   db_client_2.py
@data           :   2024/5/11
@author         :   jiangmenggui@hosonsoft.com
"""
import re

import pymysql
from dbutils.pooled_db import PooledDB
from pymysql.cursors import DictCursor
from sshtunnel import SSHTunnelForwarder

from lljz_tools import logger


class MySQLConnectionPool:

    def __init__(self, uri: str, ssh_uri='', show_sql: bool = False):
        database_config = self._init_uri(uri)
        database_config['cursorclass'] = pymysql.cursors.DictCursor
        database_config['charset'] = 'utf8mb4'
        self._ssh = None
        self.show_sql = show_sql

        if ssh_uri:
            ssh_config = self._init_ssh_uri(ssh_uri)
            self._ssh = self._get_tunnel_ssh(
                database_config['host'], database_config['port'],
                ssh_config['host'], ssh_config['port'], ssh_config['user'], ssh_config['password']
            )
            database_config['host'] = '127.0.0.1'
            database_config['port'] = self._ssh.local_bind_port
            database_config['autocommit'] = True

        # 创建连接池
        self._pool = PooledDB(
            creator=pymysql,
            # 其他参数根据你的应用需求调整
            mincached=1,
            maxcached=5,
            maxconnections=10,
            blocking=True,  # 如果没有可用连接，调用者将等待而不是抛出异常
            **database_config
        )

    @staticmethod
    def _init_uri(uri):
        obj = re.match(
            r'^mysql://(?P<user>.+):(?P<password>.+)@(?P<host>.+):(?P<port>\d+)/(?P<db>.+?)(?:\?(?P<qs>.+))?$',
            uri
        )
        if not obj:
            raise ValueError(f'unmatch uri: {uri}')
        database_config = obj.groupdict()
        if database_config['qs']:
            qs = {k: v for k, v in (i.split('=', maxsplit=1) for i in database_config.pop('qs').split('&'))}
            database_config.update(qs)
        else:
            database_config.pop('qs')
        database_config['port'] = int(database_config['port'])
        return database_config

    @staticmethod
    def _init_ssh_uri(ssh_uri):
        obj = re.match(f'^ssh://(?P<user>.+):(?P<password>.+)@(?P<host>.+?)(?::(?P<port>[0-9]+))?$', ssh_uri)
        if not obj:
            raise ValueError(f'unmatch ssh_uri: {ssh_uri}')
        ssh_config = obj.groupdict()
        if 'port' not in ssh_config:
            ssh_config['port'] = 22
        ssh_config['port'] = int(ssh_config['port'])
        return ssh_config

    @staticmethod
    def _get_tunnel_ssh(host, port, ssh_ip, ssh_port, ssh_user, ssh_pwd):
        # private_key = paramiko.RSAKey.from_private_key_file(
        #     filename=ssh_key_path,
        #     password=ssh_pwd
        # )
        ssh_tunnel = SSHTunnelForwarder(
            ssh_address_or_host=(ssh_ip, ssh_port),
            ssh_username=ssh_user,
            ssh_password=ssh_pwd,
            remote_bind_address=(host, port)
        )
        ssh_tunnel.start()
        return ssh_tunnel

    def close(self):
        self._pool.close()
        if self._ssh:
            self._ssh.close()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    def connect(self):
        return MySQLConnection(self._pool.connection(), self.show_sql)


class MySQLConnection:

    def __init__(self, conn, show_sql: bool):
        self.conn = conn
        self._show_sql = show_sql

    def close(self):
        self.conn.close()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    def show_sql(self, cursor: DictCursor, sql, args):
        if self._show_sql:
            try:
                logger.debug(cursor.mogrify(sql, args), stacklevel=3)
            except TypeError as e:
                logger.error(f'[ERROR SQL] sql: {sql}, args: {args}', stacklevel=3)
                logger.exception(e, stacklevel=3)

    def select(self, sql: str, args=None, limit: int = 1000):
        sql = sql.replace('?', '%s')
        with self.conn.cursor() as cursor:
            self.show_sql(cursor, sql, args)
            cursor.execute(sql, args)
            return cursor.fetchmany(limit)

    def select_one(self, sql: str, args=None):
        sql = sql.replace('?', '%s')
        with self.conn.cursor() as cursor:
            self.show_sql(cursor, sql, args)
            cursor.execute(sql, args)
            return cursor.fetchone()

    def select_all(self, sql, args=None):
        sql = sql.replace('?', '%s')
        with self.conn.cursor() as cursor:
            self.show_sql(cursor, sql, args)
            cursor.execute(sql, args)
            return cursor.fetchall()

    def insert(self, sql: str, args):
        sql = sql.lstrip()
        sql_type, _ = sql.split(maxsplit=1)
        if sql_type.upper() != 'INSERT':
            raise ValueError(f'NOT INSERT SQL: {sql}')
        sql = sql.replace('?', '%s')
        with self.conn.cursor() as cursor:
            self.show_sql(cursor, sql, args)
            cursor.execute(sql, args[0])
            return cursor.lastrowid

    def insert_many(self, sql: str, args):
        sql = sql.lstrip()
        sql_type, _ = sql.split(maxsplit=1)
        if sql_type.upper() != 'INSERT':
            raise ValueError(f'NOT INSERT SQL: {sql}')
        sql = sql.replace('?', '%s')
        with self.conn.cursor() as cursor:
            for arg in args:
                self.show_sql(cursor, sql, arg)
            cursor.executemany(sql, args)
            return cursor.rowcount

    def update(self, sql: str, *args: tuple | list):
        sql = sql.lstrip()
        sql_type, _ = sql.split(maxsplit=1)
        if sql_type.upper() != 'UPDATE':
            raise ValueError(f'NOT UPDATE SQL: {sql}')
        sql = sql.replace('?', '%s')
        with self.conn.cursor() as cursor:
            self.show_sql(cursor, sql, args)
            cursor.execute(sql, args)
            return cursor.rowcount

    def delete(self, sql: str, *args):
        sql = sql.lstrip()
        sql_type, _ = sql.split(maxsplit=1)
        if sql_type.upper() != 'DELETE':
            raise ValueError(f'NOT DELETE SQL: {sql}')
        sql = sql.replace('?', '%s')
        with self.conn.cursor() as cursor:
            self.show_sql(cursor, sql, args)
            cursor.execute(sql, args)
            return cursor.rowcount

    def commit(self):
        self.conn.commit()

    def rollback(self):
        self.conn.rollback()


if __name__ == '__main__':
    pass




import setuptools

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()


kwargs = dict(
    name="lljz_tools",
    version="0.3.1",
    author="liulangjuanzhou",
    author_email="liulangjuanzhou@gmail.com",
    description="常用工具封装",
    long_description_content_type="text/markdown",
    url="",
    package_data={
        "my_tools": ["*.html", "*.js"]
    },
    install_requires=[
        # 'pymysql>=1.1.0', 'dbutils>=3.0.3', 'sshtunnel>=0.4.0',
        # 'requests>=2.31.0', 'urllib3>=2.1.0',
        'openpyxl>=3.1.2', 'colorlog>=6.8.2',
        'better-exceptions>=0.3.3', 'concurrent-log-handler>=0.9.23'
    ],
    classifiers=[
        "Programming Language :: Python :: 3.12",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
setuptools.setup(
    **kwargs,
    long_description=long_description,
    packages=setuptools.find_packages(),
)



# coding=utf-8

"""
@fileName       :   excel.py
@data           :   2024/2/22
@author         :   jiangmenggui@hosonsoft.com
"""
import os.path

from openpyxl import load_workbook, Workbook
from openpyxl.worksheet.worksheet import Worksheet


class ExcelReader:

    def __init__(self, excel_filename: str):
        if not excel_filename.endswith('.xlsx'):
            raise TypeError('Only support xlsx file')
        if not os.path.exists(excel_filename):
            raise FileNotFoundError(f"{excel_filename} not found!")
        self._file = excel_filename
        self._excel = self._get_excel()

    def _get_excel(self):
        return load_workbook(self._file, read_only=True, data_only=True)

    def _get_sheet(self, sheet: int | None | str) -> Worksheet:
        if isinstance(sheet, int):
            return self._excel.worksheets[sheet]
        elif isinstance(sheet, str):
            return self._excel[sheet]
        return self._excel.active

    def read(self, sheet: int | None | str = None, /, *, has_title: bool = True, min_row=None, max_row=None,
             min_col=None, max_col=None):
        """
        读取Excel的Sheet页中的数据

        如果没有指定索引，则范围从A1开始。如果工作表中没有单元格，则返回一个空元组。

        :param sheet: sheet页的顺序或名字或默认
        :type sheet: int | None | str

        :param has_title: 是否包含标题，如果包含标题则每行数据都会以字典的形式返回，否则以元组的形式返回
        :type has_title: bool

        :param min_col: 最小列索引(从1开始)
        :type min_col: int

        :param min_row: 最小行索引(从1开始)
        :type min_row: int

        :param max_col: 最大列索引(从1开始)
        :type max_col: int

        :param max_row: 最大行索引(从1开始)
        :type max_row: int

        :rtype: generator
        """
        sheet = self._get_sheet(sheet)
        iter_rows = sheet.iter_rows(min_row, max_row, min_col, max_col, values_only=True)
        if has_title:
            title = next(iter_rows)
            yield from (dict(zip(title, r)) for r in iter_rows)
        else:
            yield from iter_rows

    @property
    def sheet_names(self):
        return self._excel.sheetnames

    def close(self):
        self._excel.close()


class ExcelWrite:

    def __init__(self, excel_filename: str):
        if not excel_filename.endswith('.xlsx'):
            raise TypeError('Only support xlsx file')
        self._file = excel_filename
        self._sheets: dict[str, Worksheet] = {}
        self._excel = self._get_excel()

    def _get_excel(self) -> Workbook:  # noqa
        return Workbook(write_only=True)

    def write(self, data: list[list | dict], /, *, sheet_name=None) -> Worksheet:
        if not sheet_name:
            sheet_name = f'Sheet{len(self._sheets) + 1}'
        if sheet_name not in self._sheets:
            self._sheets[sheet_name] = self._excel.create_sheet(title=sheet_name)
        if not data:
            return self._sheets[sheet_name]

        data = iter(data)
        first = next(data)
        if isinstance(first, dict):
            self._sheets[sheet_name].append(tuple(first.keys()))  # write_title
            self._sheets[sheet_name].append(tuple(first.values()))
        else:
            self._sheets[sheet_name].append(first)
        for r in data:
            if isinstance(r, dict):
                self._sheets[sheet_name].append(tuple(r.values()))
            else:
                self._sheets[sheet_name].append(r)
        return self._sheets[sheet_name]

    def save(self):
        self._excel.save(self._file)

    def close(self):
        self._excel.close()


class Excel(ExcelReader, ExcelWrite):

    def __init__(self, excel_filename: str):
        super().__init__(excel_filename=excel_filename)
        self._sheets = {name: self._excel[name] for name in self._excel.sheetnames}

    def _get_excel(self):  # noqa
        return load_workbook(self._file, data_only=True)

    def write(self, data, /, *, sheet_name=None):
        if not sheet_name:
            sheet_name = self._excel.active.title
        return super().write(data, sheet_name=sheet_name)

    def __getitem__(self, item) -> Worksheet:
        return self._excel[item]


if __name__ == '__main__':
    # excel = Excel(r"./test.xlsx")
    # sheet = excel['Sheet2']
    # print(sheet)
    # for row in excel.read(has_title=True):
    #     print(row)
    # print()
    # excel.write([{"name": 'Jack', 'age': 77}])
    # excel.save()
    pass
